///  <reference path="vendor/toastr.js" />
///  <reference path="syncor.js" />
/// <reference path="/js/vendor/knockout-2.2.1.js" />
/// <reference path="vendor/jquery-1.9.1.min.js" />
/// <reference path="~/js/viewModel.js" />
/*global $, vm, toastr, syn, ko, _machineState */
//Note no space before comment for JSLint, JSHint wants the space
//==============================================================================
// JavaScript for NetBurner Support
// DESCRIPTION: This file contains the basic code showing how to write state
// data from the NetBurner to a JavaScript object. A couple of techniques are used 
// including NetBurner's FunctionCall, but mostly it uses AJAX.
// In this version the KnockoutJs library is binding data to the page. Most
// of the code that was in this file in earlier versions has been removed since 
// jQuery does a lot of what this code used to do. 
// The reference paths at the top are an IntelliSense aid for Visual Studio and
// ReSharper.
// Copyright 2009-2013 Tod Gentille Syncor Systems,Inc.
//See syncor.js for applicable licensing information.
//==============================================================================

//==============================================================================
// Executable code that runs when the page that includes this file loads.
//==============================================================================
var shell = shell || {};

(function () {
    "use strict";
    var req_object = {
        Version: '1.0.0',
        // Constants for Making Ajax Requests
        REQUEST_CMD: "request",
        MACHINE_STATE: "machineState",
        SAMPLE_DATA: "sampleData",
        RESULT_DATA: "resultData",
        DROPDOWN_OPTIONS: "dropdownOptions"
    },
        ticks_between_callbacks = 4;

    if (syn === 'undefined') {
        throw ("shell requires the syn library from syncor.js");
    }

    if (vm === 'undefined') {
        throw ("shell requires the vm library from viewModel.js");
    }

    shell.sample_data = {};
    shell.TICK_ID = "tick";
    shell.RANDOM_ID = "random";
    shell.SAMPLE_DROP_DOWN_ID = "sampleDropDownBox";
    shell.HIDE_CLASS = "hidden";
    shell.KNOCKOUT_PAGE = "#knockoutPage";
    shell.BASIC_AJAX_PAGE = "#basicAjaxPage";
    shell.ABOUT_PAGE = "#aboutPage";
    shell.LINK = "Link";
    shell.pageDivs = [shell.BASIC_AJAX_PAGE, shell.KNOCKOUT_PAGE, shell.ABOUT_PAGE];


    //==============================================================================
    // When the form loads set up all the GUI elements to reflect the current
    // state of the NetBurner, then start the Clock that triggers the  AJAX
    // callbacks
    //==============================================================================
    function doLoad() {
        try {
            shell.addSampleSelect();
            shell.setupCheckboxes();
            shell.setupSampleSelectViaAjax();
            shell.StartClock();
            shell.createDebugDropDown();
        } catch (ex) {
            toastr.error(ex.message);
            console.log(ex.message);
        }
    }

    //==============================================================================
    // The jQuery onReady event. Everything from the onLoad event (see doLoad) could
    // now be moved here. Ready fires sooner than on load (although some elements 
    // might not be loaded, like images). I'm leaving doLoad in just because this is
    // a demo database and it shows the old-fashioned approach to adding and event
    // handler.
    // We set up some tooltips and the code to trigger when the navbar links are
    // clicked. 
    //==============================================================================
    $(function () {
        //The second parameter is optional (and in this case redundant), 
        //but it you want multiple viewmodels on a page you just pass the part of the page
        //you want each view model to apply to. see http://knockoutjs.com/documentation/observables.html
        ko.applyBindings(vm.demoViewModel, window.document.body);

        $('#koLegendLeft').tooltip({ placement: 'right' });
        $('#koLegendRight').tooltip({ placement: 'left' });
        $("a[data-rel='tooltip']").tooltip();
        $('#koResultData').tooltip({ placement: 'top' });

        toastr.info("jQuery onReady finished");
        $(shell.KNOCKOUT_PAGE + shell.LINK).click(function () {
            shell.doPageLinks(shell.KNOCKOUT_PAGE);
        });
        $(shell.BASIC_AJAX_PAGE + shell.LINK).click(function () {
            shell.doPageLinks(shell.BASIC_AJAX_PAGE);
        });
        $(shell.ABOUT_PAGE + shell.LINK).click(function () {
            shell.doPageLinks(shell.ABOUT_PAGE);
        });
        doLoad();
    });

    //==============================================================================
    // The three "pages" of the app are really just divs that get hidden or shown
    // This is one way to start on a single page app (SPA). You can also use Ajax to load
    // html from the server into sections if you want to keep the main page pretty
    // lean and clean. The Ajax html loads would get cached so after the first visit
    // to each page it would have the responsiveness of doing it this way.
    // A real SPA would also track what page the user was last one, enable the back
    // forward buttons and more but this example is already complicated enough.
    //==============================================================================
    shell.doPageLinks = function (pageToShow) {
        $(pageToShow).removeClass(shell.HIDE_CLASS);

        $.each(shell.pageDivs, function (index, value) {
            if (value !== pageToShow) {
                var elem = $(value);
                elem.addClass("hidden");
            }
        });
    };

    //==============================================================================
    // Set the checkbox based on the  _machineState setting
    //==============================================================================
    shell.setupCheckboxes = function () {
        vm.demoViewModel.testCheckbox(window._machineState.checkBoxState);
    };

    //Create the debug drop down by setting the data in the view model and then let
    //Knockout's data binding create the options. Set the selected option.
    shell.createDebugDropDown = function () {
        var ms = window._machineState;
        vm.demoViewModel.debugOptions(ms.debugOptions);
        vm.demoViewModel.debugValue(ms.debugOptions[ms.debugState]);
    };

    //==============================================================================
    // When the timer expires it ends up calling this function.
    //==============================================================================
    shell.CallbackNetburner = function () {
        var param_obj = {},
            request;
        param_obj[req_object.REQUEST_CMD] = req_object.SAMPLE_DATA;
        request = $.param(param_obj);
        syn.MakeAjaxRequestJson(request, shell.CreateSampleData);
        //We aren't limited to making just one request
        param_obj[req_object.REQUEST_CMD] = req_object.RESULT_DATA;
        request = $.param(param_obj);
        syn.MakeAjaxRequestJson(request, shell.CreateResultData);
    };

    //==============================================================================
    // Show Ajax updating by getting a random number and a time tick value from the 
    // NetBurner. Put it in two different places in the page. 
    //==============================================================================
    shell.CreateSampleData = function (resultJson) {
        if (resultJson === null || resultJson === undefined) {
            return;
        }
        //Put it into the html of a div using jQuery
        var description = syn.DescribeObject(resultJson);
        $('#objectSample').html(description);
        //Put it in the the view model and let knockout's binding put it in the table.
        vm.demoViewModel.systemTick(resultJson.systemTick);
        vm.demoViewModel.randomValue(resultJson.randomValue);
        $("#ajaxTest").hide(400);
    };


    shell.CreateResultData = function (resultjson) {
        vm.demoViewModel.addResult(resultjson.resultData);
        //If you returned an array of data you would just do this:
        //$.each(resultjson.resultData, function() {
        //    vm.demoViewModel.addResult(this);
        //});
    };

    //==============================================================================
    // Set up a timer for the callback to the NetBurner
    //==============================================================================

    shell.timerTicks = ticks_between_callbacks;
    shell.StartClock = function () {
        shell.timerTicks -= 1;
        setTimeout(shell.StartClock, 200); // delay is in milliseconds
        if (shell.timerTicks === 0) {
            shell.timerTicks = ticks_between_callbacks;
            shell.CallbackNetburner();
        }
    };

    shell.addSampleSelect = function () {
        var node_id = shell.SAMPLE_DROP_DOWN_ID,
            drop_down_list = syn.CreateSelectSequential(10, 1);
        $("#" + node_id).empty().append(drop_down_list);
    };

    shell.setupSampleSelectViaAjax = function () {
        var request = syn.MakeUrlParmList(req_object.REQUEST_CMD, req_object.DROPDOWN_OPTIONS);
        syn.MakeAjaxRequestJson(request, shell.CreateSampleSelect);
    };


    //Some JavaScript that builds a select from scratch. Using Knockout is MUCH easier
    shell.CreateSampleSelect = function (resultData) {
        var selectResponse = resultData,
            selectDomObj = syn.CreateSelect(selectResponse.options);
        selectDomObj.selectedIndex = selectResponse.selectedItem;
        $("#sampleAjaxDropDownBox").empty().append(selectDomObj);
    };
}());
