//==============================================================================
//	Copyright (C) 2006 Syncor Systems, Inc.
//                    25500 Hawthorne Blvd. Ste 1158
//                    Torrance, CA 90505
//                      http://www.syncorsystems.com
// General Purpose JavaScript Support Library Includes support
// for AJAX and JSON communication exchange. Relies on the jQuery library.
// 
//
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:

//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//SOFTWARE.
//==============================================================================

/*global $, toastr */
/* global $, toastr */
//^^jsHint requires the leading space jsLint requires no leading space

//Create a pseudo namespace to hold all the helper methods
var syn = syn || {};

(function () {

    "use strict";

    if ($ === undefined) {
        throw ("syn requires the jQuery library.");
    }
    if (toastr === undefined) {
        throw ("syn requires the toastr library");
    }


    //==============================================================================
    // Create a Select element that is composed of sequential numbers
    // @param numDesired - the number of options the select area should have
    //==============================================================================
    syn.CreateSelectSequential = function (numDesired, startValue) {
        var the_select = document.createElement("select"),
            i,
            the_option;

        for (i = 0 ; i < numDesired; ++i) {
            the_option = new Option(i + startValue, i + startValue); //display , value both set to i
            the_select.options[i] = the_option;
        }
        return the_select;
    };


    //==============================================================================
    // Creates an XHTML <select> element  from the passed array of properties, 
    // @param optionList an array of value and display properties 
    // @returns a fully valid select element from the DOM
    //==============================================================================
    syn.CreateSelect = function (optionList) {
        try {
            var the_select = document.createElement("select"),
                i,
                the_option;

            for (i = 0 ; i < optionList.length; ++i) {
                the_option = new Option(optionList[i].display, optionList[i].value);
                //If you wanted an option to show up in the list but have it disabled you could do
                //something like this to disable the third item.
                //if (i == 3) the_option.disabled = true;
                the_select.options[i] = the_option;
            }
            return the_select;
        }
        catch (exception) {
            console.log("syn.CreateSelect: " + exception.message);
        }
        return null;
    };


    //==============================================================================
    // Ajax code using the jquery library. This is the only method needed
    // for doing Ajax calls. The Ajax object and specifically the Ajax.Request
    // method allows us to specify both the request and the function to be called
    // on completion. I wrote this wrapper so that the completion function could
    // be passed in and therefore used by all requests in the entire system. 
    // In V3 I added the data type of "json" so that I no longer have to "eval" the
    // results into a JSON object, jQuery does it for me but it is more strict about
    // having a well formed JSON object, specifically all names in the name/value pair
    // must be enquoted.
    //==============================================================================

    syn.MakeAjaxRequestJson = function (theParams, completionFunction, callingScope) {
        var pass_scope = false;
        if (callingScope) {
            pass_scope = true;
        }
        try {

            $.ajax("ajax.htm", {
                type: "GET",
                data: theParams,
                cache: false,
                dataType: "json",
                success: function (resultData/*, status, xhrResp*/) {
                    if (pass_scope) {
                        completionFunction.call(callingScope, resultData);
                    }
                    else {
                        completionFunction(resultData);
                    }
                },
                error: function (jqxhrObj, txtStatus, errorThrown) {
                    toastr.error(errorThrown, txtStatus);
                    console.log("syncor.js:MakeAjaxRequestJson-"+txtStatus + "-" + errorThrown);
                    console.log("Ajax Response Text:" + jqxhrObj.responseText);
                }

            }).done(function () { });

        }
        catch (exception) {          
            toastr.error(exception.message, "syn.MakeAjaxRequest: ");
            console.log("syn.MakeAjaxRequest threw:" + exception.message);
        }
    };


    //==============================================================================
    // A very simplistic way to show the name/value pairs in an object. If the value
    // is an object it recurses. If the object is an Array it doesn't, because Arrays are 
    // rather verbose objects and we just want to see a list of the array values and 
    // the standard value will give us that. Arrays that hold arrays of Objects will
    // just return the [Object object] string so any calling methods that have 
    // arrays of objects will have to handle that themselves. This method is intended
    // for early debugging of JSON strings that are passed back from the instrument.
    // This allows a quick way to see what's in the JSON object that was created. 
    // @param theObj - the JavaScript object we want to examine - typically created
    // be eval()ing a JSON string but it doesn't have to be. 
    // @returns the full description string
    //==============================================================================
    syn.DescribeObject = function (theObj) {
        var description = "",
            property_name,
            property_value;
        for (property_name in theObj)  //just step through every property in the object
        {
            //skip any prototypal additions to the object
            if (!theObj.hasOwnProperty(property_name)) {
                continue;
            }

            property_value = theObj[property_name]; //use the indexer to get the property value

            if (property_value instanceof Array) //std op for an Array
            {
                description += property_name + "=" + property_value + "<br />";
            }
                //if it's an object recurse (otherwise it would just return [Object])
            else if (property_value instanceof Object) {
                description += property_name + ": " + property_value + "<br />";
                if (property_name.indexOf("extend") < 0) {
                    description += syn.DescribeObject(property_value);
                }
            } else //in our JSON spec there are no other types, other than simple, so just get the value
            {
                description += property_name + "=" + property_value + "<br />";
            }
        }
        return description;
    };



    //==============================================================================
    // This method will take any number of arguments and turn them into a URL
    // parameter string of the form "?name=value&name2=value2. It is expected that 
    // if the user wants back a fully valid string they will pass in an even
    // number of arguments.
    // @parameter any number of string arguments
    // @returns the url parameter list string
    //==============================================================================
    syn.MakeUrlParmList = function () {
        var url_param_list = "",
            sep = "=",
            i,
            max_args;

        for (i = 0, max_args = arguments.length; i < max_args; ++i) {
            url_param_list += arguments[i];
            if (i < (max_args - 1)) {
                url_param_list += sep; //add separator on all but last param
            }
            sep = (sep === "=") ? "&" : "=";  //toggle url argument separator between = and &
        }
        return url_param_list;
    };

}());
