//
// HtmlServices.cpp
//
//  Created on: Nov 18, 2009
//      Author: Tod
//

#include "HtmlServices.h"

#include <iostream>
#include <sstream>
#include <utils.h> //has TimeTick
#include <string>
#include <iosys.h> //for writestring
//#include <constants.h>
#include <htmlfiles.h>
//#include "ShellCommon.h"
#include <http.h>
#include <map>
#include "JsonCreator.h"
#include <vector>
#include "../A2D.h"

#include "Debug.h"

using namespace std;

//==============================================================================
// This page holds the C++ code that is exposed to the HTML pages for use
// as callbacks. We minimize the callbacks by using javascript when possible.
// One of the primary purposes of this code is to write the machine state out
// to the web page using a javascript object with properties. Then the javascript
// can analyze that info to customize the HTML to reflect the machine state.
//==============================================================================

HtmlServices::AjaxFunctionMap HtmlServices::AjaxCommands;
const string HtmlServices::_requestToken = "request";

bool HtmlServices::_testCheckbox = true;

HtmlServices::HtmlServices()
{
    AjaxCommands.insert(make_pair(_requestToken + "=machinestate", &WriteJsonMachineState));
    AjaxCommands.insert(make_pair(_requestToken + "=sampleData", &WriteSampleJsonData));
    AjaxCommands.insert(make_pair(_requestToken + "=dropdownOptions", &WriteDropDownOptionsJsonData));
//    AjaxCommands.insert(make_pair(_requestToken+"=sampleUiData", &WriteSampleUiData));
    AjaxCommands.insert(make_pair(_requestToken + "=resultData", &WriteSampleResultData));
}

double HtmlServices::GetRandomNumber()
{
    int random_number = rand();
    random_number %= 1000000;
    const double value = random_number / 1000.0;

    return value;
}

void HtmlServices::ProcessAjaxFunction(const int sock, string url)
{

    string::size_type found_string = url.find(_requestToken);
    if (found_string == string::npos)
    {
        ostringstream msg;
        msg << "Url did not contain " << _requestToken << endl;
        Debug::Write(msg.str());
        return;
    }

    string command = url.substr(found_string);
    found_string = command.find("&");
    if (found_string != string::npos)
    {
        command = command.substr(0, found_string);

    }

    const AjaxFunctionMap::const_iterator ajax_iter = AjaxCommands.find(command);
    if (ajax_iter != AjaxCommands.end())
    {
        //cout << "found a command " << ajax_iter->first << endl;
        ajax_iter->second(sock);
    }
    else
    {
        cout << "could not find match for " << command << endl;
    }

}

//==========================================================================================
//There's no particular need to write this state machine object out into a script on the page.
//It would actually be easier to have the Ajax response handler just write the returned
//json data into the view model. But this is a demo so part of the point is to show various
//ways of doing things.
//==========================================================================================
void HtmlServices::WriteJsonMachineState(const int sock)
{
    const string MACHINE_STATE_VAR = "_machineState";
    const string DEBUG_STATE_PROP = "debugState";
    const string CHECKBOX_STATE_PROP = "checkBoxState";

    JsonCreator jc;
    jc.AddProperty(DEBUG_STATE_PROP, Debug::GetLevel());
    jc.AddProperty(CHECKBOX_STATE_PROP, _testCheckbox);
    jc.PushProperty(Debug::GetDebugOptionsJson());

    //Create the first part of the string which is a JavaScript var  not JSON
    ostringstream java_script;
    java_script << "<script> var " << MACHINE_STATE_VAR << "=\n";
    //Now get the JSON object
    java_script << jc.GetJsonObject() << "</script>";
    writestring(sock, java_script.str().c_str());

    Debug::Write(java_script.str(), dl_TACITURN);
}

//==============================================================================
// Example of how to assemble a well-formed JSON object for creating a select
// drop down list.
//==============================================================================
void HtmlServices::WriteDropDownOptionsJsonData(const int sock)
{
    map<int, string> debug_options;
    map<int, string>::iterator map_iter;

    debug_options.insert(make_pair(1, "The First Item "));
    debug_options.insert(make_pair(2, "Another Item"));
    debug_options.insert(make_pair(3, "The Last Item"));

    ostringstream json_literal;
    json_literal << OBJECT_START << JsonCreator::StartArray("options");
    for (map_iter = debug_options.begin(); map_iter != debug_options.end();)
    {
        JsonCreator jc;
        jc.AddProperty("value", map_iter->first);
        ostringstream disp_value;
        disp_value << map_iter->second << ' ' << TimeTick;
        jc.AddProperty("display", disp_value.str(), ValueHandler::String);
        json_literal << jc.GetJsonObject();
        if (++map_iter != debug_options.end()) json_literal << ",";
    }
    json_literal << ARRAY_DELIM_END << OBJECT_END;
    writestring(sock, json_literal.str().c_str());
}

//==============================================================================
// Example of how to write out a well formed JSON object that jQuery can treat
// as json data and automatically turn into a JavaScript object.
// In SupportScripts.js you will see function CreateSampleData(resultData)
// that gets set as the responder to our AJAX call and it processes the data created
// here by making it the HTML of a div and by assigning it to the view model.
//==============================================================================
void HtmlServices::WriteSampleJsonData(const int sock)
{
    const string SYSTEM_TICK = "systemTick";
    const string RANDOM_VALUE = "randomValue";

    Debug::Write("J", dl_VERBOSE, mt_GREATER_THAN_OR_EQUAL);

    JsonCreator jc;
    jc.AddProperty(SYSTEM_TICK, TimeTick);
    jc.AddProperty(RANDOM_VALUE, GetRandomNumber());
    writestring(sock, jc.GetJsonObject().c_str());

}



void HtmlServices::WriteSampleResultData(const int sock)
{

    //Send back an array of objects where each object contains three pieces of data
    //Knockout will use a template to display all the data.
    const int MIN_CH = 0;
    const int MAX_CH = 7;
    static int channel = MIN_CH;
    const short a2d_value = A2D::Read(channel);
    ++channel;
    if (channel > MAX_CH) {channel =MIN_CH;}

    JsonCreator jc;
    jc.AddProperty("a2dValue", a2d_value);
    jc.AddProperty("timeTick", TimeTick);
    jc.AddProperty("randomNumber",GetRandomNumber());
    writestring(sock, jc.GetJsonObject("resultData").c_str());

}

//==============================================================================
// Post Handler - When the user submits a form this method is called
// Set up using SetNewPostHandler() via the RegisterPost() method
//==============================================================================
int HtmlServices::PostHandler(int sock, char * const url, char * const pData, char * const rxBuffer)
{
    const string REDIRECT_PAGE = "index.htm";
    cout << "In MyDoPost..." << endl;
    //In a Real application one would likely have multiple forms.
    //you would then select the proper processing function by dispatching based
    //on the url data member.
    //In this case we will ignore that....
    const int MAX_CB_RET_LEN = 3; //Checkboxes only exist in returned post if the box is enabled.
    const int MAX_DEBUG_SIZE_LEN = 16; //65536 wold be our biggest value so make it one extra
    //Declare the strings for all the HTML interface widget names
    const string TEST_CB = "ckb_test";
    const string DEBUG_CMB = "cmb_debug";

    //ExtractPostData returns -1 if field not found, 0 if found but empty
    char checkbox_data[MAX_CB_RET_LEN];
    char debug_setting[MAX_DEBUG_SIZE_LEN];

    //Checkboxes only show up in the post if they are true, so just look to see if it is here
    if (ExtractPostData(TEST_CB.c_str(), pData, checkbox_data, sizeof(checkbox_data)) != -1)
    {
        _testCheckbox = true;
    }
    else _testCheckbox = false;

    cout << "Check box is now" << _testCheckbox << '\n';

    if (ExtractPostData(DEBUG_CMB.c_str(), pData, debug_setting, sizeof(debug_setting)) != -1)
    {
        const string scpi_debug_prefix = "SYST:DEBUG  ";
        ostringstream scpi_cmd;
        scpi_cmd << scpi_debug_prefix << " " << debug_setting;
        cout << "SCPI command: " << scpi_cmd.str() << endl;
        Debug::SetLevel(debug_setting);
    }
    else
    {
        cout << "Error in extracting text from debug setting popup." << endl;
    }

    //We have to respond to the post with a new HTML page...
    //In this case we will redirect so the browser will go to that URL for the response...
    RedirectResponse(sock, REDIRECT_PAGE.c_str());

    return 0;
}

//==============================================================================
// Install the method we want to have handle form submissions
//==============================================================================
void HtmlServices::RegisterPost() const
{
    SetNewPostHandler(PostHandler);
}

