//
//JsonCreator.cpp
//
// Created on: Dec 11, 2009
//     Author: Tod
//

// See JsonTest project for the unit tests of this class and examples of usage
#include "JsonCreator.h"

using namespace std;

JsonCreator::JsonCreator()
        : _properties(vector<string>())
{
}

string JsonCreator::StartArray(const string& propName)
{
    ostringstream new_property;

    bool enquote = false;
    if (propName[0] != '\"') enquote = true;
    if (enquote) new_property << STR_DELIM;
    new_property << propName;
    if (enquote) new_property << STR_DELIM;
    new_property << NAME_VALUE_DELIM << ARRAY_DELIM_START;
    return new_property.str();
}


void JsonCreator::AddProperty(string propName, string propValue, const valueHandlerType valueType)
{
    ostringstream new_property;
    new_property << STR_DELIM << propName << STR_DELIM << NAME_VALUE_DELIM;
    new_property << BuildPropertyValue(propValue, valueType);
    _properties.push_back(new_property.str());
}



string JsonCreator::GetJsonObject(std::string optionalName)
{
    ostringstream json_object;

    json_object << OBJECT_START;
    if (optionalName.length()>0)
    {
        json_object << STR_DELIM<<optionalName <<STR_DELIM<<":{ ";
    }
    AddJsonProperties(json_object);
    if (optionalName.length()>0){ json_object << OBJECT_END;}
    json_object << OBJECT_END;
    return json_object.str();
}



string JsonCreator::GetProperties()
{
    ostringstream json_props;
    AddJsonProperties(json_props);
    return json_props.str();
}

void JsonCreator::ClearProperties()
{
    _properties.clear();
}

void JsonCreator::AddJsonProperties(ostringstream& destObj)
{
    if (_properties.empty()) return;
    vector<string>::iterator prop_iter;

    for (prop_iter = _properties.begin(); prop_iter != _properties.end();)
    {
        destObj << *prop_iter;
        if (++prop_iter != _properties.end()) destObj << PROPERTY_DELIM;
    }
}

string JsonCreator::BuildPropertyValue(string propValue, const valueHandlerType valueType) const
{
    ostringstream build_prop_value;
    switch (valueType)
    {
    case ValueHandler::Normal:
        build_prop_value << propValue;
        break;

    case ValueHandler::String:
        build_prop_value << STR_DELIM << propValue << STR_DELIM;
        break;

    default:
        break;
    }
    return build_prop_value.str();
}



