//
// JsonCreator.h
//
//  Created on: Dec 11, 2009
//      Author: Tod
//

#pragma once

#include <vector>
#include <map>
#include <string>
#include <sstream>

namespace ValueHandler
{
    enum ValueHandler
    {
        Normal, String
    };
}
typedef ValueHandler::ValueHandler valueHandlerType;

const std::string OBJECT_START = "{";
const std::string OBJECT_END = "}";
const std::string NAME_VALUE_DELIM = ": ";
const std::string STR_DELIM = "\"";
const std::string ARRAY_DELIM_START = "[";
const std::string ARRAY_DELIM_END = "]";

class JsonCreator
{
    public:
        JsonCreator();
        void AddProperty(std::string propName, std::string propValue, const valueHandlerType valueType);
        void PushProperty(std::string propObject){_properties.push_back(propObject);}

        template<typename T>
        void AddProperty(std::string propName, T propValue)
        {
            std::string new_value_str = AddValue(propValue);
            std::ostringstream new_property;
            new_property << STR_DELIM << propName << STR_DELIM << NAME_VALUE_DELIM;
            new_property << BuildPropertyValue(new_value_str, ValueHandler::Normal);
            _properties.push_back(new_property.str());
        }

        template<typename T>
        std::string AddValue( T propValue)
        {
            std::ostringstream new_value;
            new_value << propValue;
            return new_value.str();
        }

        std::string AddValue (const bool propValue) const
        {
            std::ostringstream new_value;
            new_value << std::boolalpha<< propValue;
            return new_value.str( );
        }

        static std::string StartArray(const std::string& propName);

        std::string GetJsonObject(std::string optionalName="");
        std::string GetProperties();
        void ClearProperties();

    private:
        static const char PROPERTY_DELIM = ',';
        std::vector<std::string> _properties;
        void AddJsonProperties(std::ostringstream& destObj);
        std::string BuildPropertyValue(std::string propValue, const valueHandlerType valueType) const;

};

